# DI/OT Igloo2-based Radiation-tolerant System Board

DI/OT Igloo2-based Radiation-tolerant System Board, currently at the schematics design stage.
[More info at the Wiki page](https://ohwr.org/project/diot-sb-igl/wikis/home)

How to build ?
==============

1. Clone

2. Also clone submodules

    > git submodule update --init
    
3. Be sure to have hdlmake >= 3.4dev

   (See https://ohwr.org/project/hdl-make.git, use branch develop)
   
4. Build bitstream, you need liberoSoc > 2021

    > cd diot-sb-gl/hdl/syn/diot_hydra2  
    > hdlmake  
    > make

   (You might need to update the constraints and not reserve probe pins,
    and not enforce package compatibility).

5. Optionaly, use libero GUI:

    > hdlmake  
    > make project  
    > libero

6. Be sure to have gcc for risc-v
   Could be installed from:

      https://ohwr.org/project/wrpc-sw/wikis/Documents/Project-Attachments
      
   (Click on riscv-11.2-small.tgz)
   
7. Build the embedded software:

    > cd diot-sb-gl/sw/fip_urv  
    > make

8. In order to flash, you need to build the envm project.

    > cd diot-sb-gl/hdl/syn/envm

   There is a TCL script to be run by libero.
   Then you need to configure the envm to enable flash and add the
   `fip_urv.mem` file into the flash
   
9. Flash the bitstream and the envm.


How to use ? (mini-guide)
=========================

Build mfip_urv (for linux) in sw/mfip_urv.  You need to refer to masterfip
libraries:

    make REPO_PARENT=xxx/masterfip

To run mfip_urv, the masterfip board must be up and running (drivers
must be loaded, firwmare must be loaded).  In general it requires root
permissions.  Commands can be passed on the command line:

    sudo ./mfip_urv ping

You can also run it in server mode which waits for commands on a tcp socket:

    sudo ./mfip_urv --port 2000 &

Note: there is no security (except connections are accepted only from
localhost).

You can also traces all packets exchanged:

    sudo ./mfip_urv --port 2000 -t

See `sw/mfip_urv/README.txt` for the tests.

With mfip_urv you can load an application to the user cpu, check the
platform health and exchanging data with the application.


Demo applications
=================

They are in plc_urv/ directory.

For plc0 (with shared memory and data transfer over fip):
- app_counter.c: increment a word in the data received and send the packet
- app_data_inc.c: xor and increment bytes.
- app_discard.c: just accept data

For plc0 (triplicated cpu) - old
- plc_mbox_inc.c: mbox exchange, force divergence

For plc1 (ecc based):
- plc1_mbox_not.c: mbox exchange, with tests for corruption
- plc_mbox.c: mbox exchange, test for corruption
- plc_mbox_tb.c: initialize mboxout, do nothing else.

Old demo (with relays)
- plc_demo1.c
- plc_demo2.c
- plc_demo.c
- plc_dio.c


Load an application
===================

First, halt the application cpu:

    plc 0 reset    # do a full reset
    plc 0 iwrite   # allow write to iram, clear dram

Load (it automatically starts):

    load 0 file.elf

NOTE: never do iwrite without reset, that's not reliable
(missing ack).


Replacing fip_urv firmware
==========================

That's only for developer.  The fip_urv firmware runs on the management cpu,
check platform health (power supply, incoming power, fantray, errors),
handle the FIP link and control the user cpus.  Users are not supposed to
modify this firmware.  In case of errors, you might need a jtag probe to
recover the platform.

First rebuild the software in sw/fip_urv:

    make

This should create a `fip_urv.elf` binary file.
By default, the firmware is loaded into the management cpu from ENVM at
offset 0, but using the `restart-cfg` command to can change the offset or
the boot device (ENVM or flash).  For testing a new firmware, we recommand
to load it into the flash:

    ./mfip_urv spi-flash-elf fip_urv.elf 2

(where 2 is a version number you can use to check the image).
Then you need to change the default boot configuration:

    ./mfip_urv restart-cfg 0xb

Too see the meaning of 0xb, you can display the boot configuration:

    ./mfip_urv restart-cfg

    restart config: 0x0000000b
    iram copy:  0x00000001 (/ 0x00000001)
    dram clear: 0x00000002 (/ 0x00000002)
    iram err:   0x00000000 (/ 0x00000004)
    iram src:   0x00000008 (/ 0x00000008)
    src page:   0x00000000 (/ 0x00ff0000) (addr=page*256)

Finally, you can restart the platform:

    ./mfip_urv restart now

You can check the build date using the `stat-regs` command:

    ./mfip_urv stat-regs
