/******************************************************************************/
/**
 *
 * @file fip_i2c_cmd.h
 *
 * This is the header file for i2c
 *
 ******************************************************************************/
#ifndef __FIP_I2C_CMD_H
#define __FIP_I2C_CMD_H

/***************** Macros (Inline Functions) Definitions **********************/
/* I2C open-core registers */
#define OC_I2C_PRES_LO      0x00
#define OC_I2C_PRES_HI      0x01
#define OC_I2C_CTRL         0x02
#define OC_I2C_RXTX         0x03
#define OC_I2C_CR           0x04
#define OC_I2C_CFG          0x05

/* I2c commands.
   Encoding:
   xxxx0000: idle, set-addr, run, err, readlen
   xxxx0100: set-targ
   xxxxxs01: read, read_stop
   xxxxxs10: write, write_stop
*/
#define I2C_CMD_IDLE         0x00	/* No more commands to execute */
#define I2C_CMD_RUN          0x10
#define I2C_CMD_NEXT         0x30	/* Execute next i2c command */
#define I2C_CMD_START_W      0x40
#define I2C_CMD_START_R      0x50
#define I2C_CMD_READLEN_STOP 0x60
#define I2C_CMD_SET_ADDR     0x70	/* followed by addr */
#define I2C_CMD_ERR          0xf0

#define I2C_CMD_SET_TARG     0x04	/* 4 msb: bus id + addr */

#define I2C_CMD_READ         0x01
#define I2C_CMD_READ_STOP    0x05
#define I2C_CMD_WRITE        0x02
#define I2C_CMD_WRITE_STOP   0x06

/* I2c slaves (flags in system buffer) */
#define I2C_SLV_ADC         0x01
#define I2C_SLV_FT          0x02
#define I2C_SLV_RTP         0x04

/* System Board ADC I2C Address */
#define ADC_SB_I2C_ADDR     (0x2F << 1)
#define ADC_SB_I2C_BUS_ID   0
#define ADC_SB_I2C_BYTES    18

/* Fantray Monimod I2C Address */
#define FANTRAY_I2C_ADDR    (0x12 << 1)
#define FANTRAY_I2C_BUS_ID  1
#define FANTRAY_I2C_BYTES   14

/* Ratopus AD7291 I2C Address */
#define RAD7291_I2C_ADDR    (0x2A << 1)
#define RAD7291_I2C_BUS_ID  2
#define RAD7291_I2C_BYTES   10

/* user i2c cmds lenght */
#define USER_I2C_LEN    16

#endif /* __FIP_I2C_CMD_H */
