/******************************************************************************/
/**
 *
 * @file crc.h
 *
 * This is the header file with CRC functions
 *
 ******************************************************************************/
#ifndef __ADC_PARAMETERS_H
#define __ADC_PARAMETERS_H

#ifdef __cplusplus
extern "C" {
#endif

/***************************** Include Files **********************************/
#include <stdint.h>
#include <stdio.h>

/***************** Macros (Inline Functions) Definitions **********************/
#define C_COF_ADC_TEMP_AMB      0.5
#define C_COF_ADC_TEMP_LDO      0.5
#define C_COF_ADC_TEMP_FPGA     0.5
#define C_COF_ADC_P12V0         6.0
#define C_COF_ADC_SENSE_P12V0   2.0
#define C_COF_ADC_P1V2          0.5
#define C_COF_ADC_P3V3          1.47
#define C_COF_ADC_PPERIPH       3.35

#define C_COF_RTP_P12V0         6.0
#define C_COF_RTP_P5V0          2.6

/* Used linear regresion on python from some values read */

/*
 * Calculate with function linregress in python:
 * --------------------------------------------
 *      Y = intercept + slope * X
 *
 */
#define C_COF_RTP_I12V0_SLOPE       0.0046944852886777475
#define C_COF_RTP_I12V0_INTERCEPT   -3.555014029400685

/*
 * Calculate with function linregress in python:
 * --------------------------------------------
 *      Y = intercept + slope * X
 *
 */
#define C_COF_RTP_I5V0_SLOPE        0.0037904926623568793
#define C_COF_RTP_I5V0_INTERCEPT    -2.4876465940595303

#define C_ADC7291_FRACTIONAL_BITS   2
#define C_ADC7291_SIGNED_BIT        11
#define C_ADC7291_SIGNED_VAL        (1 << (C_ADC7291_SIGNED_BIT - C_ADC7291_FRACTIONAL_BITS))

/************************** Constant Definitions ******************************/
const float C_ADC_RESOLUTION    = 4095.0;
const float C_ADC_REF_VOLTAGE   = 2.5;
const float C_RTP_REF_VOLTAGE   = 2.5;

/* 10 mv/ºC */
const float C_LM45_SCALE        = 100.0; //1000mV/10C

/* Coeficients to adjust the ADC from System Board */
const float C_COF_ADC_ARRAY[] = {
    C_COF_ADC_TEMP_AMB, C_COF_ADC_TEMP_LDO,
    C_COF_ADC_TEMP_FPGA, C_COF_ADC_P12V0,
    C_COF_ADC_SENSE_P12V0, C_COF_ADC_P1V2,
    C_COF_ADC_P3V3, C_COF_ADC_PPERIPH
};

/* Naming to plot DATA */
const char C_ADC_NAME_ARRAY[][16] = {
    "Tamb", "Tldo",
    "Tfpga", "P12V",
    "P12A", "P1.2V",
    "P3.3V", "Vperi",
};

/* Coeficients to adjust the ADC from RATOPUS */
const float C_COF_RTP_ARRAY[] = {
    C_COF_RTP_P12V0, 0.0,
    C_COF_RTP_P5V0, 0.0
};

/* Naming to plot DATA */
const char C_RTP_NAME_ARRAY[][16] = {
    "RtpP12V", "RtpP12A",
    "RtpP5V", "RtpP5A"
};

#ifdef __cplusplus
}
#endif

#endif /* __ADC_PARAMETERS_H */
