#include "common-app.h"

struct my_regs {
  unsigned pkt_counter;		/* @ 0x00, off 0 */
  unsigned recovery;		/* @ 0x04, off 1 */
  unsigned last_reset;		/* @ 0x08, off 2 */
  unsigned wd_dis;		/* @ 0x0c, off 3 */
  unsigned scratch;		/* @ 0x10, off 4 */
  unsigned divmemw[5];		/* @ 0x14, off 5 */
  unsigned char divmemb[8];	/* @ 0x28, off 10 */
  unsigned recov_dis;	  	/* @ 0x30, off 12 */
  			  	/* @ 0x34, off 13 */
};

struct my_regs mr;

void
init (void)
{
#if 0
  REGS->mailboxes.mboxout = 0x12345678;
#endif
#if 0
  /* Enable scrubber */
  SUPERVISOR->scrub_cfg = HYDRA_SUPERVISOR_REGS_SCRUB_CFG_IRAM_EN
    | HYDRA_SUPERVISOR_REGS_SCRUB_CFG_DRAM_EN;
#endif
}

void
main (void)
{
  /* Watchdog */
  if (!mr.wd_dis)
    SUPERVISOR->wd_key = WD_KEY;

  if (REGS->mailboxes.status & MBOX_REGS_STATUS_MBIN)
    {
      unsigned dat;
      unsigned val;

      dat = REGS->mailboxes.mboxin;

      if (dat >> 31) {
        unsigned cmd = (dat >> 24) & 0x7f;

        if (cmd == 0x01) {
          /* For testing: corrupt ra */
          val = (dat >> 16) & 0xff;
          /*  format: wrecc rd, rs1, ecc */
          asm volatile (".insn   r 0x5b, 0x1, 0, ra, ra, %0" : : "r"(val));
          val = dat & 0xffff;
        }
        else if (cmd == 0x02) {
          /* Command 2: corrupt zero */
          val = (dat >> 16) & 0xff;
          asm volatile (".insn   r 0x5b, 0x1, 0, zero, zero, %0" : : "r"(val));
          val = dat & 0xffff;
        }
        else if (cmd == 0x03) {
          /* Command 3: corrupt a reg, use it as rs1 */
          unsigned ecc = (dat >> 16) & 0xff;
          unsigned ival = (dat & 0xffff);
          //* wrecc ival, ival, ecc */
          asm volatile (".insn   r 0x5b, 0x1, 0, %2, %2, %1\n"
                        " nop; nop; nop\n"
                        " or %0,%2,zero" : "=r"(val) : "r"(ecc), "r"(ival));
        }
        else if (cmd == 0x04) {
          /* Command 4: corrupt a reg, use it as rs2 */
          unsigned ecc = (dat >> 16) & 0xff;
          unsigned ival = (dat & 0xffff);
          //* wrecc ival, ival, ecc */
          asm volatile (".insn   r 0x5b, 0x1, 0, %2, %2, %1\n"
                        " nop; nop; nop\n"
                        " or %0,zero,%2" : "=r"(val) : "r"(ecc), "r"(ival));
        }
        else if (cmd == 0x5) {
          /* Command 5: read my_reg */
          volatile unsigned *addr = (volatile unsigned *)&mr;
          val = addr[dat & 0xf];
        }
        else if (cmd == 0x7) {
          /* Command 7: tfunc() */
          extern unsigned tfunc(unsigned);
          val = tfunc(dat);
        }
        else if (cmd == 0x0c) {
          /* Command 12: generate a watchdog error */
          mr.wd_dis = 1;
          val = dat;
        }
        else if (cmd == 0x0d) {
          /* Command 0xd: poison scratch */
          SUPERVISOR->dram_ecc_mask = (dat >> 16) & 0x0f;
          val = dat & 0xffff;
          mr.scratch = val;
          SUPERVISOR->dram_ecc_mask = 0;
        }
        else if (cmd == 0x0f) {
          /* Command f: enable scrubber */
          val = dat & 0xff;
          SUPERVISOR->scrub_cfg = val;
        }
        else
          val = ~0;
      }
      else
        val = ~dat;

      REGS->mailboxes.mboxout = val;
#if 0
      if (dat == 0) {
        /* For testing: corrupt ra, add 3 nops to avoid bypasses.  */
        asm volatile (".insn   r 0x5b, 0x1, 0, ra, ra, %0\n"
                      " nop; nop; nop" : : "r"(1));
      }
      else if (dat == 1) {
        /* For testing: corrupt zero!.  */
        asm volatile (".insn   r 0x5b, 0x1, 0, zero, zero, %0" : : "r"(1));
      }
#endif
    }
}
