#include "common-app.h"

struct my_regs {
  unsigned pkt_counter;		/* @ 0x00, off 0 */
  unsigned recovery;		/* @ 0x04, off 1 */
  unsigned last_reset;		/* @ 0x08, off 2 */
  unsigned wd_dis;		/* @ 0x0c, off 3 */
  unsigned scratch;		/* @ 0x10, off 4 */
  unsigned divmemw[5];		/* @ 0x14, off 5 */
  unsigned char divmemb[8];	/* @ 0x28, off 10 */
  unsigned recov_dis;	  	/* @ 0x30, off 12 */
  			  	/* @ 0x34, off 13 */
};

struct my_regs mr;

void
init (void)
{
  /* Do not reset mboxout, otherwise it will be unsynchronized with
     mboxin.  */

  /* Enable scrubber */

  mr.pkt_counter = 0;
  mr.recovery = 0;
  mr.last_reset = 0;
  mr.wd_dis = 0;
}

void
main (void)
{
  unsigned mbi;
  unsigned mbo;
  unsigned rwslot;
  unsigned lrslot;
  unsigned lwslot;
  unsigned rrslot;
  unsigned lnslot;

  /* Force recovery in case of divergence */
  unsigned cpu = SUPERVISOR->cpu;
  if (!mr.recov_dis && cpu != 0) {
    mr.recovery++;
    mr.last_reset = cpu;
    asm volatile ("");
    SUPERVISOR->cpu = HYDRA_SUPERVISOR_REGS_CPU_RECOVERY;
  }

  /* Watchdog */
  if (!mr.wd_dis)
    SUPERVISOR->wd_key = WD_KEY;

  mbi = REGS->mailboxes.mboxin;  /* written by mgmt - remote*/
  mbo = REGS->mailboxes.mboxout; /* written by app - local */
  rwslot = (mbi >> 8) & 0xff; /* Remote slots (read from). */
  lrslot = mbo & 0xff;
  lwslot = (mbo >> 8) & 0xff; /* Local slots (write to).  */
  rrslot = mbi & 0xff;
  lnslot = (lwslot + 1) & 3;

  /* Something to read, and free slot to send. */
  if (lrslot != rwslot && lnslot != rrslot) {
    volatile uint32_t *src = &REGS->sh_mem[lrslot << 5];
    volatile uint32_t *dst;
    unsigned ecc_de = REGS->app_ecc_de;
    unsigned val;
    unsigned cmd;

    /* Data: 0x80 CMD KEY1 KEY2  VAL[7:0] VAL[15:8] VAL[23:16] VAL[31:24] */
    cmd = src[0];
    val = src[1];

    if (ecc_de != REGS->app_ecc_de) {
      /* Double ecc error in data.
         Cannot exploit data. */
      val = ++REGS->nbr_pkt_errs;
      cmd = 0xfd;
    }
    else if ((cmd >> 16) == 0xa11c) {
      cmd = (cmd >> 8) & 0xff;

      if (cmd < 5) {
        /* Command 0 : insn divergence,
           command 1 : data divergence,
           command 2 : daddr write divergence,
           command 3 : daddr read divergence
           command 4 : byte-enable divergence */
      }
      else if (cmd == 0x5) {
        /* Command 5: read my_reg */
        volatile unsigned *addr = (volatile unsigned *)&mr;
        val = addr[val];
      }
      else if (cmd == 0x6) {
        /* Command 6: read divmemb */
        val = mr.divmemb[val];
      }
      else if (cmd == 0x7) {
        /* Command 7: tfunc() */
        extern unsigned tfunc(unsigned);
        val = tfunc(val);
      }
      else if (cmd == 0x8) {
        /* Command 8: write my_reg */
        volatile unsigned *addr = (volatile unsigned *)&mr;
        unsigned idx = src[0] & 0x01f;
        addr[idx] = val;
        val = idx;
      }
      else if (cmd == 0x9) {
        /* Command 9: read recovery register */
        val = SUPERVISOR->cpu;
      }
      else if (cmd == 0x0c) {
        /* Command 12: generate a watchdog error */
        mr.wd_dis = 1;
      }
      else if (cmd == 0x0d) {
        SUPERVISOR->dram_ecc_mask = src[0] & 0x0f;
        mr.scratch = src[1];
        SUPERVISOR->dram_ecc_mask = 0;
      }
      else if (cmd == 0x0e) {
        val = mr.scratch;
      }
      else if (cmd == 0x0f) {
        SUPERVISOR->scrub_cfg = src[1];
      }
      else {
        /* Unknown command.  */
        cmd = 0xfe;
        val = ~0;
      }

      /* TODO: test iram/dram se */
    }
    else {
      cmd = 0xff;
      val = ~val;
    }

    mbo = (mbo + 0x01) & 0xffffff03;

    dst = &REGS->sh_mem[128 + (lwslot << 5)];

    mr.pkt_counter++;

    /* REP: 0x80 CMD 0 0 CNT[] VAL[] */
    dst[0] = 0x80 | (cmd << 8);
    dst[1] = mr.pkt_counter;
    dst[2] = val;

    mbo = (mbo + 0x100) & 0xffff03ff;

    REGS->mailboxes.mboxout = mbo;

    if (cmd == 0) {
      /* Divergence on insn addr. */
      if (SUPERVISOR->force_divergence & val)
        SUPERVISOR->wd_key = WD_KEY;
    }
    else if (cmd == 1) {
      /* Divergence on data */
      mr.divmemw[0] = SUPERVISOR->force_divergence & val;
    }
    else if (cmd == 2) {
      /* Divergence on addr (as write) */
      unsigned idx = SUPERVISOR->force_divergence & val;
      mr.divmemw[idx] = val;
    }
    else if (cmd == 3) {
      /* Divergence on addr (as read) */
      unsigned idx = SUPERVISOR->force_divergence & val;
      mr.scratch = mr.divmemw[idx];
    }
    else if (cmd == 4) {
      /* Divergence on byte-en */
      unsigned idx = SUPERVISOR->force_divergence & val;
      idx = (0x30210 >> (idx << 2)) & 3;
      mr.divmemb[idx] = val + 1;
    }
  }
}
